from nose.tools import *
from kazookid import Context


def test_context_manager():
    '''Context: How to create a Context'''
    assert_true(Context())

def test_can_enter_and_exit_a_context():
    '''Context: Can Enter a Context'''
    with Context() as context:
        assert_true(context)
        context.method()

    # Can still do the same tricks
    assert_true(context.method.was_called)
    assert_false(context.different_method.was_called)


from nose.tools import *
from kazookid import Substitute


def test_substitute():
    '''Substitute: How to create a Substitute'''
    substitute = Substitute()
    assert_false(substitute.method.was_called)


def test_call_method_returns_none():
    '''Substitute: How to call a method on a substitute'''
    substitute = Substitute()
    data = substitute.method()
    assert_equal(data, None)


def test_call_method_on_substitute():
    '''Substitute: How to verify that a method was called'''
    substitute = Substitute()
    substitute.method()
    assert_true(substitute.method.was_called)
    assert_false(substitute.other_method.was_called)


def test_call_intercepts_asserts_arguments():
    '''Substitute: How to verify that a method was called with specific arguments'''
    substitute = Substitute()
    substitute.method.returns(1)
    data = substitute.method('hello')
    assert_true(substitute.method.was_called_with('hello'))
    assert_false(substitute.method.was_called_with('bye'))


def test_call_intercepts_single_argument():
    '''Substitute: How to intercept the arguments of a call'''
    substitute = Substitute()
    data = substitute.method('hello')
    assert_equal(substitute.method.args, 'hello')


def test_call_intercepts_many_arguments():
    '''Substitute: If a call had many aruments, they are returned as a tuple'''
    substitute = Substitute()
    data = substitute.method('hello', 1, 2, 3)
    assert_equal(substitute.method.args, ('hello', 1, 2, 3))


def test_call_method_can_return_data():
    '''Substitute: How to provide data from a method'''
    substitute = Substitute()
    substitute.method.returns(1)

    data = substitute.method()
    assert_equal(data, 1)


def test_set_substitute_property():
    '''Substitute: How to provide data from a property'''
    substitute = Substitute()
    substitute.name = 'Johannes'
    assert_equal(substitute.name, 'Johannes')


def test_set_substitute_method():
    '''Substitute: Properties containing functions behave as such'''
    substitute = Substitute()
    substitute.name = lambda: 1
    assert_equal(substitute.name(), 1)


class Apology(Exception):
    pass

@raises(Apology)
def test_call_method_can_raise_an_exception():
    '''Substitute: How to raise exceptions'''
    substitute = Substitute()
    substitute.method.raises(Apology)
    substitute.method()


def test_yields():
    '''Substitute: How to provide data for iteration'''
    substitute = Substitute()
    substitute.yields(['a', 'b', 'c'])
    assert_equal(list(substitute), ['a', 'b', 'c'])


class Substitute(object):

    '''
    Substitutes are can be used to provide or verify
    direct and indirect input and output of objects
    during testing.

    See: http://xunitpatterns.com/

    Test Spy
    ========

    Test Spies capture indirect output
    http://xunitpatterns.com/Test%20Spy.html

    # Check if a method was called

        from kazookid import Substitute

        def test_spy():
            engine = Substitute()
            game = Game(engine)
            game.run()
            assert_true(engine.render.was_called)

    # Check on parameters

        def test_spy():
            engine = Substitute()
            game = Game(engine)
            game.run()
            assert_true(engine.render.was_called_with(time))


    Test Stub
    =========

    Stubs feed indirect input into the System under Test
    http://xunitpatterns.com/Test%20Stub.html

    # To provide indirect input:

        substitute.value = 5

        substitute.method.returns(1)

        substitute.method.raises(Exception)
    '''

    def __init__(self):
        self._calls = {}
        self._config = {}
        self._iterator = Iterator()

    def __getattr__(self, name):
        if name == 'yields':
            return self._iterator

        config = self.__dict__['_config']
        call = Call(name=name, parent=self)
        config[name] = config.get(name, call)
        return config[name]

    def __iter__(self):
        return self._iterator.__iter__()


class Context(Substitute):

    def __enter__(self, *args, **kwargs):
        return self

    def __exit__(self, *args, **kwargs):
        pass


class Call(object):

    '''
    Intercepts a call or method on a substitute object
    '''

    def __init__(self, name, parent):
        self.name = name
        self.parent = parent
        self.return_value = None
        self.exception = None

    def __call__(self, *args, **kwargs):
        if self.exception:
            raise self.exception()

        self.parent._calls[self.name] = (args, kwargs)
        try:
            return self.return_value[0]
        except:
            return None

    @property
    def args(self):
        args, kwargs = self.parent._calls.get(self.name, (None, None))
        if len(args) == 1:
            return args[0]
        return args

    def raises(self, exception):
        self.exception = exception

    def returns(self, *args, **kwargs):
        self.return_value = args

    @property
    def was_called(self):
        return self.name in self.parent._calls

    def was_called_with(self, *args):
        if len(args) == 1:
            args = args[0]
        return self.was_called and self.args == args


class Iterator(object):

    def __init__(self):
        self._items = []

    def __iter__(self):
        return self._items.__iter__()

    def __call__(self, items):
        self._items = items



# Thank you for helping me out, Jonathan
# http://jtushman.github.io/blog/2013/06/17/sharing-code-across-applications-with-python/
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

config = {
    'description': 'Can we do more kazoo?',
    'author': 'Johannes Hofmeister',
    'url': 'https://github.com/cessor/kazookid',
    'download_url': 'https://github.com/cessor/kazookid',
    'author_email': '',
    'version': '1.2',
    'tests_require': ['nose'],
    'packages': ['kazookid'],
    'scripts': [],
    'name': 'kazookid'
}

setup(**config)
